<#--common macros/functions for NoS reports (PDF and CSV)-->


<#--  List of IUCLID's composite entity types  -->
<#assign compositeEntityTypes = ["SUBSTANCE", "MIXTURE", "TEMPLATE"] />


<#-- 
    The isCompositeEntityDocument function checks if a component is a composite entity and, 
    if so, returns true.

    Inputs:
    - component: ENTITY to be checked
-->
<#function isCompositeEntityDocument component>
    <#return compositeEntityTypes?seq_contains(component.documentType) />
</#function>


<#-- 
    The traverseComponent macro checks if a component is either a simple or composite 
    entity (SUBSTANCE, MIXTURE, or TEMPLATE) and calls the corresponding macro to traverse 
    the entity.

    Inputs:
    - component: ENTITY to be traversed
-->
<#macro traverseComponent component>
    <#if isCompositeEntityDocument(component)>
        <@traverseCompositeEntity component />
    <#else>
        <@traverseSimpleEntity component />
    </#if>
</#macro>


<#-- 
    The traverseSimpleEntity macro iterates through the documents of a component and 
    triggers directly the getLitRefsFromNode macro on a simple entity.

     Inputs:
     - component: ENTITY to be traversed
-->
<#macro traverseSimpleEntity component>
    <#list component.documents as document>
        <@getLitRefsFromNode component document />
    </#list>
</#macro>


<#-- 
    The traverseCompositeEntity macro retrieves the table of contents of a composite 
    entity, iterates through its documents and triggers the getLitRefsFromNode macro.

    Inputs:
    - component: ENTITY to be traversed
-->
<#macro traverseCompositeEntity component>
    <#--  Get table of contents  -->
    <#local toc = iuclid.localizeTreeFor(component.documentType, component.submissionType, component.documentKey, getInheritedTemplates(component), getRelatedCategories(component), getRelatedMixtures(component)) />
    
    <#if component?has_content>
        <#list component.documents as document>
            <#--  Get node for the table of contents  -->
            <#assign tocNode = toc.nodeFor["${document.documentType}.${document.documentSubType}"] />
                                    
            <#if document?has_content>
                <@getLitRefsFromNode component document tocNode />
            </#if>
        </#list>
    </#if>
</#macro>


<#--
    The getLitRefsFromNode function retrieves all the literature references from a given document.

    Arguments:
    - component: ENTITY to be traversed
    - node: CURRENT NODE being processed
    - tocNode: NODE for the table of contents

    The function performs the following steps:
    1. Check if the "node" is of type "document_reference". If so, it's a direct reference to a document.
       - If the referenced document exists and is of type "LITERATURE", get parent document, NoSIDs and 
       section and append this info to either NoSstudyHash or missingNoSstudyHash.
    2. Check if the "node" is of type "document_references". If so, it's a list of document references.
       - Iterate through the list of references, and for each referenced document:
         - If the referenced document exists and is of type "LITERATURE", get parent document, NoSIDs and 
       section and append this info to either NoSstudyHash or missingNoSstudyHash.
    3. If the "node" has child sections (subsections), recursively call the function on each child section.
-->
<#macro getLitRefsFromNode component node tocNode="">
    <#--  Check if node is a document reference  -->
    <#if node?node_type == "document_reference">
        <#--  Get referenced document  -->
        <#local refDoc = iuclid.getDocumentForKey(node) />

        <#--  Check if the referenced document has content and is of type "LITERATURE"  -->
        <#if refDoc?has_content && refDoc.documentType == "LITERATURE">
            <@appendNoSStudyToHash node tocNode refDoc />
        </#if>
    </#if>

    <#--  Check if the document is a list of document references  -->
    <#if node?node_type == "document_references">
        <#--  Iterate through list of references  -->
        <#list node as item>
            <#--  Get referenced document  -->
            <#local refDoc = iuclid.getDocumentForKey(item) />

            <#--  Check if the referenced document has content and is of type "LITERATURE"  -->
            <#if refDoc?has_content && refDoc.documentType == "LITERATURE">
                <#--  Append NoS study to hash  -->
                <@appendNoSStudyToHash node tocNode refDoc />
            </#if>
        </#list>
    </#if>

    <#--  Check if document has child sections (subsections)  -->
    <#if node?children?has_content>
        <#--  Iterate through child sections  -->
        <#list node?children as child>
            <#--  Recursively call the function on each child section to retrieve literature references  -->
            <@getLitRefsFromNode component child tocNode />
        </#list>
    </#if>
</#macro>


<#--
    The getParentDocFromNode function retrieves the parent document from a given node.

    Arguments:
    - node: NODE for which to find the parent document.
-->
<#function getParentDocFromNode node>
    <#--  Iterate through node's ancestors  -->
    <#list node?ancestors as ancestor>
        <#--  Check if ancestor's node type is "document"  -->
        <#if ancestor?node_type == "document">
            <#--  Get parent document  -->
            <#return ancestor />
        </#if>
    </#list>
</#function>


<#--
    The getSectionFromNode function retrieves the section title from a given table of contents node.

    Arguments:
    - node: Table of Contents NODE from which to generate the section title.
-->
<#function getSectionFromNode tocNode>
    <#local sect = "" />

    <#if tocNode?has_content>
        <#if tocNode.number?has_content>
            <#local sect = tocNode.number + " ">
        </#if>

        <#local sect = sect + tocNode.title>
    </#if>

    <#return sect />
</#function>


<#--  
    The appendNoSStudyToHash macro appends NoS study information to two different hashes based on the 
    presence of a NoS ID in the reference document.

    Inputs:
    - node: NODE from which to get the parent document.
    - tocNode: Table of Contents (TOC) NODE from which to get the section title.
    - refDoc: REFERENCED DOCUMENT from which to extract NoS IDs info from.
-->
<#macro appendNoSStudyToHash node tocNode refDoc>
    <#--  Get parent document from node  -->
    <#local parentDoc = getParentDocFromNode(node) />
    
    <#--  Get section  -->
    <#local sect = getSectionFromNode(tocNode) />

    <#--  Look for NoSIDs in reference document  -->
    <#local NoSid = nos.getNoSid(refDoc) />

    <#--  If NoS ID exists, append data to NoSstudyHash  -->
    <#if NoSid?has_content>
        <#global NoSstudyHash = nos.addNosStudyAsUnique(refDoc, parentDoc, sect, NoSstudyHash) />
    <#--  
        If it does not exist, then append the study to the list without NoS ID if:
            - type is study report
            - year >= 2021 (or empty)
    -->
    <#else>
        <#--  Get reference type  -->
        <#local refType><@com.picklist refDoc.GeneralInfo.LiteratureType /></#local>

        <#--  Get reference year  -->
        <#local refYear = nos.getRefYear(refDoc) />

        <#--  Append data to missingNoSstudyHash -->
        <#if refType == "study report" && ((!refYear?has_content) || refYear >= 2021)>
            <#global missingNoSstudyHash = nos.addNosStudyAsUnique(refDoc, parentDoc, sect, missingNoSstudyHash) />
        </#if>
    </#if>
</#macro>


<#-- addNosStudyAsUnique appends a reference and a study into a pre-defined hashMap.
    Inputs:
    - reference: LITERATURE entity
    - document: IUCLID document e.g. ENDPOINT STUDY RECORD
    - section: string containing the section where the document is found e.g. 2.1 Appearance
    - hash: hash to add studies to
        - Keys correspond to reference uuids
        - Values correspond to another hash containing the keys "reference", "section", and "doc", each
        one containing a list of literature references, section names and studies, respectively

        Hash structure:
        { ref uuid 1 : {'reference': [reference1, reference2, ...],
                           'section': [section1, section2, ...],
                           'doc': [study1, study1, ...]},
            ref uuid 2: {..}
        }
    Outputs:
    - a HASH with structure as above, containing the additional study
-->
<#function addNosStudyAsUnique reference document section hash>

    <#local uuid = reference.documentKey.uuid/>

    <#-- if reference exists, add new document to its hash entry; else create a new entry-->
    <#if hash?keys?seq_contains(uuid)>

        <#local hashEntry = hash[uuid]/>
        <#local docs = hashEntry['doc']/>
        <#local sects = hashEntry['section']/>

        <#-- avoid duplicates! only add document if it's not already present-->
        <#local dup=false/>
        <#list docs as doc>
            <#if doc.documentKey==document.documentKey>
                <#local dup=true>
                <#break>
            </#if>
        </#list>
        <#if !dup>
            <#local sects = sects + [section]/>
            <#local docs = docs + [document]/>
        </#if>

    <#else>

        <#local docs=[document]/>
        <#local sects=[section]/>

    </#if>

    <#local hashEntry = { uuid : {'reference': reference, 'section': sects, 'doc': docs}}/>
    <#local hash = hash + hashEntry/>

    <#return hash/>

</#function>


<#function getInheritedTemplates entityDoc>
<#--
<#if !(mode?has_content && mode == "PUBLISHING") && entityDoc.documentType == "SUBSTANCE">
  <#return entityDoc.inherited>
</#if>
-->
    <#return []>
</#function>


<#function getRelatedCategories entityDoc>
    <#if !(mode?has_content && mode == "PUBLISHING")>
        <#local params={"key": [entityDoc.documentKey]}>
        <#return iuclid.query("iuclid6.SubstanceRelatedCategories", params, 0, 100)>
    </#if>
    <#return []>
</#function>


<#function getRelatedMixtures entityDoc>
<#--
<#if !(mode?has_content && mode == "PUBLISHING")>
  <#local params={"key": [entityDoc.documentKey]}>
  <#return iuclid.query("iuclid6.SubstanceRelatedMixtures", params, 0, 100)>
</#if>
 -->
    <#return []>
</#function>


<#-- getNoSid extracts an NoS ID from a literature reference entity.
    A NoS id is identified if entries under Other study identifiers have type "Notification of Studies (NoS) ID"
    and id matches or contains the format for EFSA NoS id.
    Inputs:
    - reference: LITERATURE entity
    - exactMatch: if true, it requires EFSA's NoS id format contained in the text, otherwise it retrieves the content
    of the field Study ID as it is.
    Outputs:
    - a STRING containing unique NoS IDs separated by "; "
-->
<#function getNoSid reference exactMatch=true>

    <#local NoSIds=[]/>

    <#if reference.GeneralInfo.StudyIdentifiers?has_content>
        <#list reference.GeneralInfo.StudyIdentifiers as studyId>
            <#local idType><@com.picklist studyId.StudyIDType/></#local>
            <#local idValue><@com.text studyId.StudyID/></#local>

            <#if idValue?has_content>
				<#if idType=="Notification of Studies (NoS) ID">
				    <#if exactMatch>
                        <#if idValue?matches(".*EFSA-[0-9]{4}-[0-9]{8}.*", "s")>
                            <#-- NOTE: needs the 's' flag in order to also cope with multi-line!-->
                            <#if !NoSIds?seq_contains(idValue)>
                                <#local NoSIds = NoSIds + [idValue]/>
                            </#if>
                        </#if>
	                <#else>
	                	<#if !NoSIds?seq_contains(idValue)>
	                        <#local NoSIds = NoSIds + [idValue]/>
	                    </#if>
	                </#if>
                </#if>
            </#if>
        </#list>
    </#if>

    <#if NoSIds?has_content>
        <#return NoSIds?join("; ")/>
    <#else>
        <#return ""/>
    </#if>
</#function>


<#function getDocumentUrl document>
    <#local generatedUrl = iuclid.webUrl.entityView(document.documentKey) />
    <#if generatedUrl?has_content>
    <#-- Get the base URL part -->
        <#local generatedUrl = generatedUrl?keep_before("/iuclid6-web") />
    </#if>
    <#return generatedUrl + "/iuclid6-web/?key=" + document.documentKey>
</#function>


<#macro dossierHeaderPaths dossier>
    <#compress>
        <#--  Get context and paths  -->
        <#if dossier.documentSubType == "EU_PPP_MAXIMUM_RESIDUE_LEVELS">
            <#global app = dossier.MRLApplication.DossierSpecificInformation />
            <#global nosinfo = dossier.NotificationOfStudies />
        <#elseif dossier.documentSubType=="EU_PPP_MICROORGANISMS_FOR_MIXTURES" || dossier.documentSubType=="EU_PPP_ACTIVE_SUBSTANCE_FOR_MIXTURES">
            <#global app = dossier.ActiveSubstanceApproval />
            <#global nosinfo = dossier.NotificationOfStudies />
        <#elseif dossier.documentSubType == "EU_PPP_BASIC_SUBSTANCE">
            <#global app = dossier.BasicSubstanceApproval />
            <#global nosinfo = dossier.PreApplicationInformation />
        <#elseif dossier.documentSubType == "EU_PPP_PESTICIDE_PRODUCT">
            <#global app = dossier.ApplicationForOrAmendmentToPlantProtectionProductAuthorisation />
            <#global nosinfo = "" />
            <#--NOTE: nosinfo would raise error if run with this context (not possible at moment)-->
            <#--NOTE: not considered for the moment. Fields:-->
            <#--ApplicationForOrAmendmentToPlantProtectionProductAuthorisation.ApplicationIsMadeThroughARepresentativeOfTheApplicant (picklist)-->
            <#--ApplicationForOrAmendmentToPlantProtectionProductAuthorisation.AuthorisationPreviouslyGrantedForPlantProtectionProduct (picklist)-->
            <#--ApplicationForOrAmendmentToPlantProtectionProductAuthorisation.ConclusionSOfTheMemberStateAssessingEquivalence (picklist)-->
        </#if>
    </#compress>
</#macro>


<#-- getNoSidRemarks extracts the remarks corresponding to Notification of Studies from a literature reference entity.
    A NoS remark is identified if entries under Other study identifiers have type "Notification of Studies (NoS) ID"
    and the remarks field has content.
    Inputs:
    - reference: LITERATURE entity
    Outputs:
    - a STRING containing unique remarks separated by "; "
-->
<#function getNoSidRemarks reference>

    <#local remarks=[]/>

    <#if reference.GeneralInfo.StudyIdentifiers?has_content>
        <#list reference.GeneralInfo.StudyIdentifiers as studyId>
            <#local idType><@com.picklist studyId.StudyIDType/></#local>
            
			<#if idType=="Notification of Studies (NoS) ID">
                <#local remark><@com.text studyId.Remarks/></#local>

                <#if remark?has_content>
                    <#if !remarks?seq_contains(remark)>
                        <#local remarks = remarks + [remark]/>
                    </#if>
                </#if>
            </#if>
        </#list>
    </#if>

    <#if remarks?has_content>
        <#return remarks?join("; ")/>
    <#else>
        <#return ""/>
    </#if>

</#function>


<#-- getSynonyms extracts a list of all synonyms of a reference substance.
    Inputs:
    - referenceSubstanceID: REFERENCE_SUBSTANCE entity
    Outputs:
    - a LIST containing all synonyms found
-->
<#function getSynonyms referenceSubstanceID>
    <#local synonymsList=[]>
    <#if referenceSubstanceID?has_content && referenceSubstanceID.Synonyms.Synonyms?has_content>
        <#list referenceSubstanceID.Synonyms.Synonyms as synonyms>
            <#if synonyms?has_content>
                <#local syn>
                    <#compress>
                        <#-- NOTE: if "other:" the synonym name is empty-->
                        <#if synonyms.Identifier?has_content><@com.picklist synonyms.Identifier/>: </#if>
                        <#if synonyms.Name?has_content><@com.text synonyms.Name/></#if>
<#--                        <#if synonyms.Remarks?has_content> (<@com.text synonyms.Remarks/>)</#if>-->
                    </#compress>
                </#local>
                <#local synonymsList=synonymsList+[syn]/>
            </#if>
        </#list>
    </#if>
    <#return synonymsList/>
</#function>

<#-- sanitizeUUID sanitises the UUID in order to keep just the second part after the '/',
     which corresponds to the dossier (first part is dataset).
     Input:
     - uuidPath: path to the uuid to be sanitised.
     Output:
     - a STRING with the sanitised uuid

     NOTE: to be moved to macros_common_general.ftl!
-->
<#function sanitizeUUID uuidPath>
	
	<#local uuid><@com.text uuidPath/></#local>
	<#if uuid?matches(".{2,50}/.{2,50}", "r")>
		<#local uuid=uuid?replace(".*/", '', 'r')/>
	</#if>
	<#return uuid/>
</#function>

<#-- getRefYear extracts the year from a LITERATURE entity as an integer, if possible, else
    it returns an empty string
     Input:
     - reference: LITERATURE entity
     Outut:
     - an INTEGER (number) containg the literature entity year or an empty STRING

     NOTE: for future, maybe return 0 if there is no year or year is not convertible to number??
-->
<#function getRefYear reference>

	<#local refYear><@com.number reference.GeneralInfo.ReferenceYear/></#local>
    	 
    <#if refYear?has_content>
		<#attempt>
			<#local refYear=refYear?number/>
		<#recover>
			<#local refYear=""/>
		</#attempt>
    </#if> 
    
    <#return refYear/>
</#function>

<#-- getNosInfo returns the path to the section in the dossier header where notification of studies information is
    present, depending on the dossier context.
     Input:
     - dossier: _dossierHeader variable, obtained by the default initializeMainVariables macro.
     Outut:
     - a PATH to the section in the dossier header for NoS or an empty STRING if not found.
-->
<#function getNosInfo dossier>

    <#--get context and paths-->
    <#if dossier.documentSubType=="EU_PPP_MAXIMUM_RESIDUE_LEVELS">
        <#return dossier.NotificationOfStudies/>
    <#elseif dossier.documentSubType=="EU_PPP_MICROORGANISMS_FOR_MIXTURES" ||
            dossier.documentSubType=="EU_PPP_ACTIVE_SUBSTANCE_FOR_MIXTURES">
        <#return dossier.NotificationOfStudies/>
    <#elseif dossier.documentSubType=="EU_PPP_BASIC_SUBSTANCE">
        <#return dossier.PreApplicationInformation/>
    <#else>
        <#return ""/>
    </#if>
</#function>

<#-------------------------------------------------
    OLD approach macros and functions
    (to be deleted once report is changed to traversal)
-------------------------------------------------->

<#--
    populateNoSstudyHashes parses the table of contents of a _subject document (MIXTURE, SUBSTANCE) and performs
    the actions defined in the "section_tree_node" macro below for each node.
    In case of MIXTUREs, it also parses recursively parses the tocs of all their SUBSTANCE or MIXTURE
    components as defined in the mixture composition documents and metabolites as defined in the metabolites document, and performs the same actions
-->
<#macro populateNoSstudyHashes subject>

    <#local products=[subject] + com.getOtherRepresentativeProducts(subject)/>
    <#local components=[]/>
    <#list products as prod>
        <#local components=components + com.getComponents(prod)/>
    </#list>
    <#local metabolites=com.getMetabolites(subject)/>

    <#--Remove duplicates-->
    <#local allEntities = products/>
    <#list components+metabolites as sub>
        <#local allEntities=com.addDocumentToSequenceAsUnique(sub, allEntities)/>
    </#list>

    <#--Parse-->
    <#list allEntities as entity>
        <#local toc=iuclid.localizeTreeFor(entity.documentType, entity.submissionType, entity.documentKey)/>
        <#recurse toc/>
    </#list>

</#macro>

<#--
    This macro is automatically applied by the #recurse function.

    It loops through the documents on a node, and for each ENDPOINT_STUDY_RECORD
    or any other documents specified in the variable 'otherDocs' (e.g. FLEXIBLE_RECORD.IntermediateEffects) it:
    - extracts all the literature references
    - appends each reference and the study where it is used into two different hashMaps depending on
    the existence of a NoS ID
    The path for the reference in the document is either the standard for ENDPOINT_STUDY_RECORDs, or must be
    hardcoded in the "otherDocs" variable.
-->
<#macro "section_tree_node">

    <#--NOTE: these other docs have all literature references, but most likely the only relevant is Intermediate Effects-->
    <#local otherDocs = {"FLEXIBLE_RECORD.IntermediateEffects":["DataSource.Reference"],
    						"FLEXIBLE_SUMMARY.AnalyticalProfileOfBatches":["AdministrativeDataSummary.BatchAnalysis"], <#-- this is a list! -->
	                        "FLEXIBLE_RECORD.ProtectionMeasures":["AdditionalInformation.Reference"],
                        	"FLEXIBLE_SUMMARY.SummaryEvaluation_EU_PPP":["OtherReferencesIncludingSDS.References"],
                        	"FLEXIBLE_RECORD.LiteratureSearch":["RelevantStudies.LiteratureReference"] <#--  also in EvaluationOfTheReview.ExcludedPublications as rep block, but should never be a study-->,
                        	"FLEXIBLE_RECORD.BioPropertiesMicro":
                        		["BiologicalPropertiesOfTheMicroorganism.GeneralInformationOnTheMicroorganism.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.GeneralInformationOnTheMicroorganism.HistoricalBackground.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.GeneralInformationOnTheMicroorganism.HistoricalUses.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.GeneralInformationOnTheMicroorganism.OriginNaturalOccurrenceAndGeographicalDistribution.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.DevelopmentStagesLifeCycleOfTheMicroorganism.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.RelationshipsToKnownPlantOrAnimalOrHumanPathogens.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.GeneticStabilityAndFactorsAffectingIt.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.InformationOnTheProductionOfMetabolitesEspeciallyToxins.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.ProductionAndResistanceToAntibioticsAndOtherAntimicrobialAgents.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.RobustnessToEnvironmentalFactors.Reference",
                        		"BiologicalPropertiesOfTheMicroorganism.FurtherInformationOnTheMicroorganism.Reference",
                        		"EffectivenessAgainstTargetOrganisms.InfectivenessDispersalAndColonisationAbility.Reference",
                        		"EffectivenessAgainstTargetOrganisms.MethodsToPreventLossOfVirulenceOfSeedStockOfTheMicroorganism.Reference",
                        		"MeasuresNecessaryToProtectHumansAnimalsAndTheEnvironment.MonitoringPlan.Reference",
                        		"BiologicalPropertiesOfTheMicroorganismInTheBiocidalProduct.Reference"],
                            "FLEXIBLE_RECORD.AnalyticalInformation":["AnalyticalInformation.DataSource.Reference"]
    }/>

    <#local contents=(.node.content)!/>

    <#-- get section -->
    <#local sect = "" />
    <#if .node?has_content>
        <#if .node.number?has_content>
            <#local sect = .node.number + " ">
        </#if>
        <#local sect = sect + .node.title>
    </#if>

    <#if contents?has_content>
        <#list contents as doc>
            <#local docTypeSubtype>${doc.documentType}.${doc.documentSubType}</#local>

            <#if (doc.documentType=="ENDPOINT_STUDY_RECORD" || otherDocs?keys?seq_contains(docTypeSubtype)) && doc?has_content>

                <#--get path-->
                <#local refPaths=[]/>
                <#if otherDocs?keys?seq_contains(docTypeSubtype)>
                	<#if docTypeSubtype=="FLEXIBLE_SUMMARY.AnalyticalProfileOfBatches">
                		<#if doc.AdministrativeDataSummary.BatchAnalysis?has_content>
                			<#list doc.AdministrativeDataSummary.BatchAnalysis as item>
                				<#if item.Reference?has_content>
                					<#local refPaths = refPaths + [item.Reference]/>
                				</#if>
                			</#list>
                		</#if>
                	<#else>
	                	<#local refPathList = otherDocs[docTypeSubtype]>
	                    <#list refPathList as refPathEntry>
	                    	<#local refPath = "doc." + refPathEntry/>
	                    	<#local refPaths = refPaths + [refPath?eval]/>
	                    </#list>
                	</#if>
                <#else>
                    <#local refPaths=[doc.DataSource.Reference]/>
                </#if>

                <#--check if document has a valid reference for NoS id, and retrieve it-->
                <#list refPaths as refPath>
	                <#local reference=getStudyReference(refPath)/>

	                <#if reference?has_content>
	                    <#list reference as ref>
	                       <#--check for NoSid based on specific criteria defined in macro-->
	                        <#local NoSid=getNoSid(ref)/>

							<#-- if NoS ID exists, append to list  -->
	                        <#if NoSid?has_content>
	                            <#assign NoSstudyHash=addNosStudyAsUnique(ref, doc, sect, NoSstudyHash)/>
	                        <#-- if it does not exist, then append the study to the list without NoS ID if:
	                        - type is study report
	                        - year >= 2021 (or empty)
	                        -->
	                        <#else>
	                            <#local refType><@com.picklist ref.GeneralInfo.LiteratureType/></#local>
	                            <#local refYear=getRefYear(ref)/>

	                            <#if refType=="study report" && ((!refYear?has_content) || refYear>=2021)>
	                                <#assign missingNoSstudyHash=addNosStudyAsUnique(ref, doc, sect, missingNoSstudyHash)/>
	                            </#if>
	                        </#if>
	                    </#list>
	                </#if>
	             </#list>
            </#if>

        </#list>
    </#if>

    <#recurse/>
</#macro>


<#--
    This function extracts a list of all the literature references from a study,
    provided a path to find the references.
    No other criteria (type of lit ref, etc) are used for filtering at the moment.
-->
<#function getStudyReference referencePath>

    <#local reference=[]/>

    <#if referencePath?has_content>
        <#list referencePath as referenceLink>
            <#local referenceEntry = iuclid.getDocumentForKey(referenceLink)/>
            <#local reference= reference + [referenceEntry]/>

        </#list>
    </#if>

    <#return reference>
</#function>